@@ -234,7 +234,11 @@ static bool_t tc_get_item(TupleEvalContext *ctx,
     }
   } else {
     while (*s != '\0' && *s != tmpendch && i < (max - 1)) {
-      if (*s == '\\') s++;
+      if (*s == '\\') {
+        s++;
+        if (*s == '\0')
+          break;
+      }
       buf[i++] = *(s++);
     }
   }
@@ -271,8 +275,11 @@ static int tc_get_variable(TupleEvalContext *ctx, char *name, int type)
 
   if (type != TUPLE_VAR_CONST) {
     for (int i = 0; i < ctx->len; i ++)
-      if (! strcmp (GET_VAR (ctx, i)->name, name))
+    {
+      TupleEvalVar * var = GET_VAR (ctx, i);
+      if (var->type == type && ! strcmp (var->name, name))
         return i;
+    }
   }
 
   return tuple_evalctx_add_var(ctx, name, type, ctype);
@@ -285,7 +292,6 @@ static bool_t tc_parse_construct(TupleEvalContext *ctx, TupleEvalNode **res,
   char tmps1[MAX_STR], tmps2[MAX_STR];
   bool_t literal1 = TRUE, literal2 = TRUE;
 
-  (*c)++;
   if (tc_get_item(ctx, c, tmps1, MAX_STR, ',', &literal1, "tag1", item)) {
     (*c)++;
     if (tc_get_item(ctx, c, tmps2, MAX_STR, ':', &literal2, "tag2", item)) {
@@ -359,31 +365,16 @@ static TupleEvalNode *tuple_compiler_pass1(int *level, TupleEvalContext *ctx, co
             break;
 
           case '=': c++;
-            if (*c != '=') {
-              /* Definition */
-              literal = FALSE;
-              if (tc_get_item(ctx, &c, tmps1, MAX_STR, ',', &literal, "variable", item)) {
-                c++;
-                if (*c == '"') {
-                  /* String */
-                  c++;
-                } else if (g_ascii_isdigit(*c)) {
-                  /* Integer */
-                }
-
-                tuple_error(ctx, "Definitions are not yet supported!\n");
-                goto ret_error;
-              } else
-                goto ret_error;
-            } else {
-              /* Equals? */
-              if (!tc_parse_construct(ctx, &res, item, &c, level, OP_EQUALS))
-                goto ret_error;
-            }
+            if (*c != '=') goto ret_error;
+            c++;
+            /* Equals? */
+            if (!tc_parse_construct(ctx, &res, item, &c, level, OP_EQUALS))
+              goto ret_error;
             break;
 
           case '!': c++;
-            if (*c != '=') goto ext_expression;
+            if (*c != '=') goto ret_error;
+            c++;
             if (!tc_parse_construct(ctx, &res, item, &c, level, OP_NOT_EQUALS))
               goto ret_error;
             break;
@@ -427,11 +418,10 @@ static TupleEvalNode *tuple_compiler_pass1(int *level, TupleEvalContext *ctx, co
               } else
                 goto ret_error;
             } else
-              goto ext_expression;
+              goto ret_error;
             break;
 
           default:
-          ext_expression:
             /* Get expression content */
             c = expr;
             literal = FALSE;
@@ -455,35 +445,15 @@ static TupleEvalNode *tuple_compiler_pass1(int *level, TupleEvalContext *ctx, co
         tuple_error(ctx, "Expected '{', got '%c' in '%s'.\n", *c, c);
         goto ret_error;
       }
-
-    } else if (*c == '%') {
-      /* Function? */
-      item = c++;
-      if (*c == '{') {
-        gssize i = 0;
-        c++;
-
-        while (*c != '\0' && (g_ascii_isalnum(*c) || *c == '-') && *c != '}' && *c != ':' && i < (MAX_STR - 1))
-          tmps1[i++] = *(c++);
-        tmps1[i] = '\0';
-
-        if (*c == ':') {
-          c++;
-        } else if (*c == '}') {
-          c++;
-        } else if (*c == '\0') {
-          tuple_error(ctx, "Expected '}' or function arguments in '%s'\n", item);
-          goto ret_error;
-        }
-      } else {
-        tuple_error(ctx, "Expected '{', got '%c' in '%s'.\n", *c, c);
-        goto ret_error;
-      }
     } else {
       /* Parse raw/literal text */
       gssize i = 0;
-      while (*c != '\0' && *c != '$' && *c != '%' && *c != '}' && i < (MAX_STR - 1)) {
-        if (*c == '\\') c++;
+      while (*c != '\0' && *c != '$' && *c != '}' && i < (MAX_STR - 1)) {
+        if (*c == '\\') {
+          c++;
+          if (*c == '\0')
+            break;
+        }
         tmps1[i++] = *(c++);
       }
       tmps1[i] = '\0';
