@@ -738,35 +738,6 @@ EXPORT void audgui_list_update_rows (GtkWidget * list, int at, int rows)
     gtk_tree_path_free (path);
 }
 
-static void move_cursor_away (GtkWidget * list, ListModel * model, int at, int rows)
-{
-    GtkTreePath * path = NULL;
-    gtk_tree_view_get_cursor ((GtkTreeView *) list, & path, NULL);
-
-    if (! path)
-        return;
-
-    int row = gtk_tree_path_get_indices (path)[0];
-
-    gtk_tree_path_free (path);
-
-    if (row < at || row >= at + rows)
-        return;
-
-    if (at + rows < model->rows)
-        row = at + rows;
-    else
-        row = at - 1;
-
-    if (row >= 0)
-        path = gtk_tree_path_new_from_indices (row, -1);
-    else
-        path = gtk_tree_path_new ();
-
-    gtk_tree_view_set_cursor ((GtkTreeView *) list, path, NULL, FALSE);
-    gtk_tree_path_free (path);
-}
-
 EXPORT void audgui_list_delete_rows (GtkWidget * list, int at, int rows)
 {
     ListModel * model = (ListModel *) gtk_tree_view_get_model
@@ -782,9 +753,23 @@ EXPORT void audgui_list_delete_rows (GtkWidget * list, int at, int rows)
     model->frozen = TRUE;
     model->blocked = TRUE;
 
-    /* prevent a warning when GTK+ tries to move the cursor to a deleted row */
-    move_cursor_away (list, model, at, rows);
+    int focus = audgui_list_get_focus (list);
 
+    // first delete rows after cursor so it does not get moved to one of them
+    if (focus >= at && focus + 1 < at + rows)
+    {
+        GtkTreePath * path = gtk_tree_path_new_from_indices (focus + 1, -1);
+
+        while (focus + 1 < at + rows)
+        {
+            gtk_tree_model_row_deleted ((GtkTreeModel *) model, path);
+            rows --;
+        }
+
+        gtk_tree_path_free (path);
+    }
+
+    // now delete rows preceding cursor and finally cursor row itself
     GtkTreePath * path = gtk_tree_path_new_from_indices (at, -1);
 
     while (rows --)
